#import "MyApplication.h"
#import "MyDocument.h"
#import <CoreMIDI/MIDIServices.h>

static void midi_read_proc (const MIDIPacketList *packets, void *refCon, void *connRefCon)
{
	int i;
	const MIDIPacket *packet;
	MyApplication *theApplication = (MyApplication*)refCon;
	
	//Step through each packet
	packet = packets->packet;
	
	
	// All Presets are asked
	// JJ : What happens if sending all presets is made from the Cybertwin ?
	if ( [ theApplication isAskedForAllPresets ] )
	{
		for (i = 0 ; i < packets->numPackets; i++) 
		{
			if ( (packet->length == 55)
				&& (packet->data[0] == 0xF0)  	//Exclusive message
				&& (packet->data[1] == 0x08)	// Fender Id
				&& (packet->data[2] == 0x21)	// Priphrique Omni
				&& (packet->data[3] == 0x11)    	// CyberTwin id
				&& (packet->data[4] == 0x02))	// Preset message
			{
				Byte midiData[packet->length];
        
				for (i=0;i<packet->length;i++)
				{
					midiData[i]=packet->data[i];
				}
				
				NSDocumentController *dc = [ NSDocumentController sharedDocumentController ];
				[ dc newDocument: nil ];
				[ theApplication sendToMyDocument : midiData ];
			}

        
			packet = MIDIPacketNext (packet);
		}
	
		[ theApplication setAskedForAllPresets: NO ];
	}
	else
	for (i = 0 ; i < packets->numPackets; i++) 
	{
		if (packet->data[0] == 176 ) 
		{
			[ theApplication sendToMyDocument : packet->data[1] : packet->data[2] ];
		}
        else
        if ( (packet->length == 55)
            && (packet->data[0] == 0xF0)  	//Exclusive message
            && (packet->data[1] == 0x08)	// Fender Id
            && (packet->data[2] == 0x21)	// Priphrique Omni
            && (packet->data[3] == 0x11)    	// CyberTwin id
            && (packet->data[4] == 0x02))	// Preset message
        {
            Byte midiData[packet->length];
        
            for (i=0;i<packet->length;i++)
            {
                midiData[i]=packet->data[i];
            }
			
			[ theApplication sendToMyDocument : midiData ];
		}

        
        packet = MIDIPacketNext (packet);
    }
    

}

@implementation MyApplication

- (id) init
{
	NSNotificationCenter *nc; 

	if (self = [ super init ])
	{
		nc = [ NSNotificationCenter defaultCenter ];
		[ nc addObserver: self  selector:@selector(receiveAllDataNotification:)  name:@"receiveAllData"  object:nil ];
		[ nc addObserver: self  selector:@selector(sendAllDataNotification:)  name:@"sendAllData"  object:nil ];
		[ nc addObserver: self  selector:@selector(sendDataNotification:)  name:@"sendData"  object:nil ];
	}
	
	
	askForAllPresets = NO;
	
	return self;
}

-(void)setAskedForAllPresets:(Boolean)value
{
	askForAllPresets = value;
}

-(Boolean)isAskedForAllPresets
{
	return askForAllPresets;
}

//Connect to the selected midi devices (in and out)
- (IBAction)connect:(id)sender
{
	if ([connectionButton state] == NSOnState)
	{
		// First we change the state of the Connect Button
		[ connectionButton setTitle: @"Disconnect" ];
		
		int inputPort = 0;
		int outputPort = 0;

		if (sender)
		{
			inputPort = [ midiInput indexOfSelectedItem ];
			outputPort = [ midiOutput indexOfSelectedItem ];
		}
			
		// We open midi port
				MIDIClientCreate (CFSTR ("CyberTwinIn"), NULL, NULL, &midiClient);

		// We open midi input port
				MIDIInputPortCreate(midiClient, CFSTR ("InputPort"), midi_read_proc, (void*)self, &midiInPort);
				currentMidiInEndpoint = MIDIGetSource (inputPort);
				MIDIPortConnectSource (midiInPort, currentMidiInEndpoint, NULL);
	  
		// We open midi output port
				MIDIOutputPortCreate(midiClient, CFSTR ("OutputPort"), &midiOutPort);
				currentMidiOutEndpoint = MIDIGetDestination (outputPort);
				MIDIPortConnectSource (midiOutPort, currentMidiOutEndpoint, NULL);

		// If AutoReceive is selected, we ask for all data
				if ([ autoReceive state ] == NSOnState)
					[ self receiveAllData ];
	}
	else
	{
		[ connectionButton setTitle: @"Connect" ];
//JJ A recopier pour le QUIT
		MIDIPortDisconnectSource(midiInPort, currentMidiInEndpoint);
		MIDIPortDisconnectSource(midiOutPort, currentMidiOutEndpoint);    
		MIDIClientDispose(midiClient);
	}
}

// send a message to the CyberTwin in order to get all presets of the CyberTwin (the returned message is analyzed by midi_read_proc function)
- (IBAction)receiveAllPresets:(id)sender
{
		askForAllPresets = YES;

        Byte buffer[1024];
        MIDIPacketList *pktlist = (MIDIPacketList*)buffer;
        MIDIPacket *curPacket=MIDIPacketListInit(pktlist);
        Byte noteOn[] = { 0xF0, 0x08, 0x00, 0x11, 0x04, 0x22, 0x00, 0x00, 0x00, 0x03, 0x7B, 0xF7 };
        curPacket = MIDIPacketListAdd(pktlist, sizeof(buffer), curPacket, 0, 12, noteOn);

        MIDISend(midiOutPort, currentMidiOutEndpoint, pktlist);
}

- (IBAction)checkDoubledBlooms:(id)sender
{
    int 	resultWindow;
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
	
	[oPanel setCanChooseFiles : NO ];
	[oPanel setCanChooseDirectories : YES ];
	
	[oPanel setAllowsMultipleSelection:NO];
   
    ///JJ Later, check the preference file
	/*    if (currentDirectory) // If wehave yet open a file, we use its directory as the default one
        resultWindow = [oPanel runModalForDirectory:currentDirectory file:nil  types:fileType ];
    else*/	// else we use the home directory
	resultWindow = [oPanel runModalForDirectory:NSHomeDirectory() file:nil  types:nil ];

    [ doubledBloomsWindow  makeKeyAndOrderFront: self];


    if (resultWindow == NSOKButton) 
    {
        NSArray *filesToOpen = [oPanel filenames];

        NSString *directoryName = [ [NSString alloc ] initWithString: [filesToOpen objectAtIndex:0] ];

		NSDirectoryEnumerator *direnum = [[NSFileManager defaultManager]
				enumeratorAtPath:directoryName];
		
		NSMutableArray *cytFiles = [ NSMutableArray arrayWithCapacity : 50 ];
		[ cytFiles setArray : [direnum allObjects] ];
		
		int i =0;
		while ( i < [ cytFiles count ])
		{
			if (! [ [ [ cytFiles objectAtIndex: i] pathExtension ] isEqualToString:@"cyt"])
				[cytFiles removeObjectAtIndex: i];
			else
				i++;
		}


		while ( [ cytFiles count ] > 1)
		{
	
			NSString *contentOfFile1 = [ NSString stringWithContentsOfFile: [NSString stringWithFormat: @"%@/%@", directoryName, [ cytFiles objectAtIndex:0] ] ];
					
			
			i = 1;
			Boolean duplicated = NO;
			while ( i < [ cytFiles count ])
			{ 
				
				NSString *contentOfFile2 = [ NSString stringWithContentsOfFile: [NSString stringWithFormat: @"%@/%@", directoryName, [ cytFiles objectAtIndex:i] ] ];
			

				if ( [ contentOfFile1 isEqual: contentOfFile2 ])
				{
					if (!duplicated)
					{
						NSLog( [NSString stringWithFormat: @"%@", [ cytFiles objectAtIndex:0] ]);
						duplicated = YES;
					}
					NSLog( [NSString stringWithFormat: @"		%@", [ cytFiles objectAtIndex:i] ]);
					[cytFiles removeObjectAtIndex: i];
				}
				else
					i++;
			}
			
			[cytFiles removeObjectAtIndex: 0];
			
		}
		
    }

}

-(void)receiveAllDataNotification:(NSNotification*)note
{
	[ self receiveAllData ];
}


// send a message to the CyberTwin in order to get all data of the preset
- (void)receiveAllData
{
        Byte buffer[1024];
        MIDIPacketList *pktlist = (MIDIPacketList*)buffer;
        MIDIPacket *curPacket=MIDIPacketListInit(pktlist);
        Byte noteOn[] = { 0xF0, 0x08, 0x00, 0x11, 0x04, 0x22, 0x00, 0x00, 0x00, 0x02, 0x7B, 0xF7 };
        curPacket = MIDIPacketListAdd(pktlist, sizeof(buffer), curPacket, 0, 12, noteOn);

        MIDISend(midiOutPort, currentMidiOutEndpoint, pktlist);
}

// send data from CyberTwin to MyDocument (in this case: realtime effects)
-(void)sendToMyDocument:(int)midiCommand :(int)midiValue
{
// Getting the main Document
	NSDocumentController *current = [ NSDocumentController sharedDocumentController ];
	MyDocument *mainDocument = [ current currentDocument ];

	if ([ autoReceive state ] == NSOnState)
		[ mainDocument updateGUI: midiCommand : midiValue ];
}

// send data from CyberTwin to MyDocument (in this case: manual all data sent )
-(void)sendToMyDocument:(Byte[])midiCommand
{
// Getting the main Document
	NSDocumentController *current = [ NSDocumentController sharedDocumentController ];
	MyDocument *mainDocument = [ current currentDocument ];
	
	[ mainDocument updateGUI: midiCommand ];
}

-(void)sendDataNotification:(NSNotification*)note
{
	char midiCommand[2];
	NSData *parameter = [ NSData dataWithData: [ note object ] ];
	
	[ parameter getBytes: midiCommand ];
	[ self sendData: midiCommand];
}

// send data to CyberTwin (in this case: realtime effects)
- (void)sendData:(char*)midiCommand
{
	char function = midiCommand[0];
	char value = midiCommand[1];
	
	NSLog(@"%c %c", function, value);

    if ([ autoSend state ] == NSOnState)
    {
		Byte buffer[1024];
		MIDIPacketList *pktlist = (MIDIPacketList*)buffer;
	    MIDIPacket *curPacket=MIDIPacketListInit(pktlist);
        Byte noteOn[] = { 176, function, value };
        curPacket = MIDIPacketListAdd(pktlist, sizeof(buffer), curPacket, 0, 3, noteOn);

        MIDISend(midiOutPort, currentMidiOutEndpoint, pktlist);
    }

}


-(void)sendAllDataNotification:(NSNotification*)note
{
	char midiCommand[70];
	NSData *parameter = [ NSData dataWithData: [ note object ] ];
	
	[ parameter getBytes: midiCommand ];
	[ self sendAllData: midiCommand];
}

// JJ Doit tre appel lors d'un NEW et lorsque le fichier est charg

// Send all data to the Cybertwin for example when initializing (at startup or when new file is activated) or when a file is loaded 
// ATTENTION: there is a bug in the Cybertwin: Expression Pedal and Continuous Controller can received 17 choices, but when it is sent 
// to the CyberTwin, it is coded with 4 bits (16 choices). The last one can't be selected
-(void)sendAllData:(char*)midiCommand
{
// Sending message
Byte buffer[1024];
MIDIPacketList *pktlist = (MIDIPacketList*)buffer;
MIDIPacket *curPacket=MIDIPacketListInit(pktlist);

curPacket = MIDIPacketListAdd(pktlist, sizeof(buffer), curPacket, 0, 68, midiCommand);
MIDISend(midiOutPort, currentMidiOutEndpoint, pktlist);

}

- (void)awakeFromNib
{
	// First We test the running macos version
	if (floor(NSAppKitVersionNumber) <= NSAppKitVersionNumber10_1) {
//JJ Il manque le localizable.strings
		NSRunAlertPanel(			NSLocalizedString(@"Attention", @"Description Forthcoming"), 
											NSLocalizedString(@"MacOSVersion", @"Description Forthcoming"), 
											NSLocalizedString(@"OK", @"Description Forthcoming"), 
											nil, 
											nil
									);

		[ [NSApplication sharedApplication] terminate: self ];
	}

	// Then we fill the fields with MIDI Devices (if they exist)
		int numberSources = MIDIGetNumberOfSources();
		int numberDestinations = MIDIGetNumberOfDestinations();

        if (numberSources > 0 && numberDestinations > 0)
        {
            int numInputs = MIDIGetNumberOfSources();
            int numOutputs = MIDIGetNumberOfDestinations();
         
            if (numInputs > 0 || numOutputs > 0)
            {
                int i;

                [ midiInput removeAllItems ];
                [ midiOutput removeAllItems ];

                for (i=0;i<numInputs;i++)
                {
                    CFStringRef deviceName;
                    MIDIEndpointRef endpoint = MIDIGetSource(i);
                    MIDIObjectGetStringProperty(endpoint, kMIDIPropertyName, &deviceName);
                    [ midiInput addItemWithTitle: (NSString*)deviceName ];
                }
                for (i=0;i<numOutputs;i++)
                {
                    CFStringRef deviceName;
                    MIDIEndpointRef endpoint = MIDIGetDestination(i);
                    MIDIObjectGetStringProperty(endpoint, kMIDIPropertyName, &deviceName);
                    [ midiOutput addItemWithTitle: (NSString*)deviceName ];
                }

            }
        }
		else
		{
			[ midiInput removeAllItems ];
            [ midiOutput removeAllItems ];
			[ connectionButton setEnabled: false ];
		}
		
 	// And we display the toolbar
 	[ connectionWindow makeKeyAndOrderFront: self ];
}

@end
